"""Builtin Speckle object kit."""

from specklepy.objects import (
    GIS,
    encoding,
    geometry,
    other,
    primitive,
    structural,
    units,
)
from specklepy.objects.base import Base

__all__ = [
    "Base",
    "encoding",
    "geometry",
    "other",
    "units",
    "structural",
    "primitive",
    "GIS",
]
